package blah;

import lombok.Getter;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import lombok.val;
import org.springframework.batch.core.Job;
import org.springframework.batch.core.JobParameters;
import org.springframework.batch.core.configuration.annotation.JobBuilderFactory;
import org.springframework.batch.core.configuration.annotation.StepBuilderFactory;
import org.springframework.batch.core.repository.JobExecutionAlreadyRunningException;
import org.springframework.batch.core.repository.JobInstanceAlreadyCompleteException;
import org.springframework.batch.core.repository.JobRepository;
import org.springframework.batch.core.repository.JobRestartException;
import org.springframework.context.annotation.Bean;
import org.springframework.stereotype.Service;

import java.util.concurrent.atomic.AtomicReference;

@Slf4j
@Service
@RequiredArgsConstructor
public class SimpleJobService {

  @Getter
  private final AtomicReference<String> result = new AtomicReference<>();

  private final JobRepository jobRepository;
  private final JobBuilderFactory jobBuilderFactory;
  private final StepBuilderFactory stepBuilderFactory;

  @Bean
  public Job runSimpleJob() {
    val job = jobBuilderFactory.get("FOO")

      .start(stepBuilderFactory.get("ONE").tasklet((a, b) -> {
        log.info("### STEP ONE!");
        return null;
      }).build())

      .next(stepBuilderFactory.get("TWO").tasklet((a, b) -> {
        log.info("### STEP TWO!");
        result.set("Step TWO DONE");
        return null;
      }).build())

      .build();

    try {
      job.execute(jobRepository.createJobExecution("Test one", new JobParameters()));
    } catch (JobExecutionAlreadyRunningException | JobRestartException | JobInstanceAlreadyCompleteException e) {
      log.error("exception in job!, e");
    }

    return job;
  }
}
